/************************************************************************/
/* Don't change anything below here unless you know what you're doing.  */
/************************************************************************/


/* Definitions */

/* Boolean variables */

#define TRUE 1
#define FALSE 0

/* Flags for sets of satisfying truth assignments */

#define SATISFY_TRUE 1 /* The truth of a specified variable is part of a
			  satisfying truth assignment */
#define SATISFY_FALSE 2

/* Miscellaneous */

#ifndef NULL
#define NULL 0
#endif

/* Definitions of data structures */

/* We store the (text) formulas, when input as text, as a linked list of
   characters.  This is memory inefficient and longer to examine than a
   normal string, but it allows an input string of any size; I judge that
   worth the drawbacks, at least for the time being. */

struct formula
	{
	char character;
	struct formula *next;
	};

struct set /* of truth assignments that satisfy a formula.  An entire set of
	      truth assignments should be referenced by a pointer to the data
	      structure with lowest variable index; any variable not specified
	      in the entire set may be either true or false.  */
	{
	long index; /* of variable being examined */
	long flags; /* Whether the truth (resp. falsity) of the indexed
		       variable can satisfy the formula, and whether the same
		       true and false in conjunction with the same set of
		       subsequent truth assignments satisfy the formula */
	struct set *true, *false; /* the sets of assignments of successive
				     variables which satisfy a formula for true
				     and false (respective) assignments of the
				     variable.  A null pointer indicates that
				     any set of assignments of successive
				     variables will work. */
	};

/* Declarations of functions: */

/* Utilities: */

struct formula *read_formula();
void append(struct formula *f, char c); /* Add a character to the end of a
					     formula. */
long calc_index(struct formula *f); /* For a sequence of digits, evaluate as an
				       integer. */
long empty(struct set *s); /* Tell if a set is empty */
void free_formula(struct formula *f); /* Garbage collection.  Delete all of a
					 formula/set list.  */
void free_set(struct set *s);
void output(struct set *s);
void output_f(struct formula *f);
long is_digit(char c);

/* Parts of the algorithm itself. */

struct set *complement(struct set *s);
struct set *intersection(struct set *s1, struct set *s2);
struct set *satisfy(struct formula *f);

